"use strict";
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("./index");
var lodash_1 = require("lodash");
var date_fns_1 = require("date-fns");
var Utils_1 = require("../Utils");
var fs = require('fs');
var path = require('path');
var mime = require('mime');
var JobParserServiceImpl = /** @class */ (function () {
    function JobParserServiceImpl(logger, jobsQueue, signatureDirectoryCreator) {
        this.logger = logger;
        this.jobsQueue = jobsQueue;
        this.signatureDirectoryCreator = signatureDirectoryCreator;
    }
    JobParserServiceImpl.prototype.parseJobFromArray = function (args) {
        var parsed = lodash_1.reduce(args, function (prev, curr) {
            return {
                o: lodash_1.get(curr.match('--o=([A-Z_]+)'), '1', prev.o),
                t: lodash_1.get(curr.match('--f=(.+)'), '1', prev.t),
                dblclk: prev.dblclk ? true : Utils_1.stringPredicate('--dblclk', 'i')(curr),
            };
        }, { o: null, t: null, dblclk: false });
        var tpl = {
            o: function (v) { return !lodash_1.isNil(v); },
            t: function (v) { return !lodash_1.isNil(v); },
            dblclk: function () { return true; }
        };
        if (lodash_1.conformsTo(parsed, tpl)) {
            return this.handleOperationTarget(parsed);
        }
        else {
            return Promise.reject('Bad input ' + JSON.stringify(parsed));
        }
    };
    JobParserServiceImpl.prototype.handleOperationTarget = function (parsed) {
        var _this = this;
        this.logger.info("Received '%s'", JSON.stringify(parsed));
        var isSignature = function (a) { return lodash_1.isEqual(a.o, 'SH_F'); };
        var isSignatureWithTS = function (a) { return lodash_1.isEqual(a.o, 'SH_FM'); };
        var isSignatureRemote = function (a) { return lodash_1.isEqual(a.o, 'SH_FR'); };
        var isSignatureRemoteWithTS = function (a) { return lodash_1.isEqual(a.o, 'SH_FRM'); };
        var isVerification = function (a) { return lodash_1.isEqual(a.o, 'SH_V'); };
        var isDecrypt = function (a) { return lodash_1.isEqual(a.o, 'SH_D'); };
        var isAskUser = function (a) { return isVerification(a) && a.dblclk; };
        return lodash_1.cond([
            [isAskUser, function (a) { return _this.handleAskUser(a.t); }],
            [isDecrypt, function (a) { return _this.handleDecrypt(a.t); }],
            [isSignature, function (a) { return _this.handleSignature(a.t); }],
            [isSignatureWithTS, function (a) { return _this.handleSignatureWithTS(a.t); }],
            [isSignatureRemote, function (a) { return _this.handleSignatureRemote(a.t); }],
            [isSignatureRemoteWithTS, function (a) { return _this.handleSignatureRemoteWithTS(a.t); }],
            [isVerification, function (a) { return _this.handleMultipleVerification(a.t); }],
            [
                lodash_1.stubTrue,
                function (a) {
                    return Promise.reject("Cannot proceed, invalid operation received " + JSON.stringify(a));
                },
            ],
        ])(parsed);
    };
    JobParserServiceImpl.prototype.guessFormat = function (files) {
        if (lodash_1.every(files, function (file) { return file.type === mime.getType('pdf'); })) {
            return 'PADES';
        }
        else if (lodash_1.every(files, function (file) { return file.type === mime.getType('xml'); })) {
            return 'XADES';
        }
        else {
            return 'CADES';
        }
    };
    JobParserServiceImpl.prototype.checkPermissions = function (target) {
        var _this = this;
        return this.fsStat(target).then(function (value) {
            if (value.isDir) {
                return _this.handleDir(target);
            }
            else {
                return {
                    type: 'FILE',
                    files: [value],
                    format: _this.guessFormat([value]),
                    defaultPath: '',
                };
            }
        });
    };
    JobParserServiceImpl.prototype.handleDir = function (target) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            fs.readdir(target, function (err, files) {
                if (err) {
                    _this.logger.warn(err);
                    reject(err);
                }
                Promise.all(files.map(function (fName) { return _this.fsStat(path.join(target, fName)); })).then(function (fileDescList) {
                    resolve({
                        type: 'DIRECTORY',
                        files: __spreadArrays(fileDescList.filter(function (value1) { return !value1.isDir; })),
                        format: _this.guessFormat(fileDescList),
                        defaultPath: '',
                    });
                });
            });
        });
    };
    JobParserServiceImpl.prototype.fsStat = function (pathStr) {
        this.logger.info("Checking access and properties of %s", pathStr);
        return new Promise(function (resolve, reject) {
            fs.access(pathStr, function (err) {
                if (err) {
                    reject(err);
                }
                else {
                    fs.stat(pathStr, function (err2, stats1) {
                        if (err2) {
                            reject(err2);
                        }
                        else {
                            resolve({
                                isDir: stats1.isDirectory(),
                                size: stats1.size,
                                name: path.parse(pathStr).base,
                                path: pathStr,
                                type: mime.getType(pathStr),
                                createdAt: date_fns_1.getTime(stats1.ctime),
                                updatedAt: date_fns_1.getTime(stats1.mtime),
                            });
                        }
                    });
                }
            });
        });
    };
    JobParserServiceImpl.prototype.parseCommandLineArgument = function (commandLineArgs) {
        var _this = this;
        var isAVerb = function (args) { return lodash_1.some(args, Utils_1.stringPredicate('(--o=|--f=)', 'i')); };
        var containsP7e = function (args) { return lodash_1.some(args, Utils_1.stringPredicate('p7e$', 'i')); };
        var containsTSD = function (args) { return lodash_1.some(args, Utils_1.stringPredicate('tsd$', 'i')); };
        var containsM7M = function (args) { return lodash_1.some(args, Utils_1.stringPredicate('m7m$', 'i')); };
        var containsP7s = function (args) { return lodash_1.some(args, Utils_1.stringPredicate('p7s$', 'i')); };
        var containsP7m = function (args) { return lodash_1.some(args, Utils_1.stringPredicate('p7m$', 'i')); };
        var containsTSR = function (args) { return lodash_1.some(args, Utils_1.stringPredicate('tsr$', 'i')); };
        return lodash_1.cond([
            [isAVerb, function (args) { return _this.parseJobFromArray(args); }],
            [containsP7m, function (args) { return _this.handleAskUser(lodash_1.find(args, Utils_1.stringPredicate('p7m$', 'i'))); }],
            [containsP7s, function (args) { return _this.handleAskUser(lodash_1.find(args, Utils_1.stringPredicate('p7e$', 'i'))); }],
            [containsP7e, function (args) { return _this.handleDecrypt(lodash_1.find(args, Utils_1.stringPredicate('p7e$', 'i'))); }],
            [containsM7M, function (args) { return _this.handleVerification(lodash_1.find(args, Utils_1.stringPredicate('m7m$'))); }],
            [containsTSD, function (args) { return _this.handleVerification(lodash_1.find(args, Utils_1.stringPredicate('tsd$'))); }],
            [containsTSR, function (args) { return _this.handleVerification(lodash_1.find(args, Utils_1.stringPredicate('tsr$'))); }],
            [lodash_1.stubTrue, function () { return Promise.resolve(null); }],
        ])(commandLineArgs);
    };
    JobParserServiceImpl.prototype.handleDecrypt = function (arg) {
        var _this = this;
        return this.checkPermissions(arg).then(function (res) {
            var isFile = function (r) { return lodash_1.isEqual(r.type, 'FILE'); };
            return lodash_1.cond([
                [isFile, function (r) { return Promise.resolve(lodash_1.get(r, 'files[0]')); }],
                [lodash_1.stubTrue, function () { return Promise.reject("Input file is a directory."); }],
            ])(res).then(function (value1) {
                var newVar = {
                    jobType: 'DECIPHER',
                    payload: value1,
                    priority: index_1.Priorities.HIGH,
                    unique: false,
                };
                _this.jobsQueue.add(newVar);
                return newVar;
            });
        });
    };
    JobParserServiceImpl.prototype.handleVerification = function (filename) {
        var _this = this;
        return this.checkPermissions(filename).then(function (res) {
            var isFile = function (r) { return lodash_1.isEqual(r.type, 'FILE'); };
            return lodash_1.cond([
                [isFile, function (r) { return Promise.resolve(lodash_1.get(r, 'files[0]')); }],
                [lodash_1.stubTrue, function () { return Promise.reject("Input file is a directory."); }],
            ])(res).then(function (value1) {
                var newVar = index_1.toVerifyJobCommand(value1);
                _this.jobsQueue.add(newVar);
                return newVar;
            });
        });
    };
    JobParserServiceImpl.prototype.handleAskUser = function (filename) {
        var _this = this;
        return this.checkPermissions(filename).then(function (res) {
            var newVar = {
                jobType: 'ASK_USER',
                payload: {
                    type: res.type,
                    files: __spreadArrays(res.files),
                },
                priority: index_1.Priorities.HIGH,
                unique: false,
            };
            _this.jobsQueue.add(newVar);
            return newVar;
        });
    };
    JobParserServiceImpl.prototype.handleSignature = function (t) {
        var _this = this;
        return this.checkPermissions(t)
            .then(function (res) {
            var payload = {
                payload: lodash_1.cloneDeep(res),
                jobType: 'DEVICE_SIGNATURE',
                priority: index_1.Priorities.HIGH,
                unique: false,
            };
            _this.jobsQueue.add(payload);
            return payload;
        })
            .then(function (value) { return _this.signatureDirectoryCreator.createDir(value); });
    };
    JobParserServiceImpl.prototype.handleSignatureWithTS = function (t) {
        var _this = this;
        return this.checkPermissions(t)
            .then(function (res) {
            var payload = {
                payload: lodash_1.cloneDeep(res),
                jobType: 'DEVICE_SIGNATURE_TS',
                priority: index_1.Priorities.HIGH,
                unique: false,
            };
            _this.jobsQueue.add(payload);
            return payload;
        })
            .then(function (value) { return _this.signatureDirectoryCreator.createDir(value); });
    };
    JobParserServiceImpl.prototype.handleSignatureRemote = function (t) {
        var _this = this;
        return this.checkPermissions(t)
            .then(function (res) {
            var payload = {
                payload: lodash_1.cloneDeep(res),
                jobType: 'REMOTE_SIGNATURE',
                priority: index_1.Priorities.HIGH,
                unique: false,
            };
            _this.jobsQueue.add(payload);
            return payload;
        })
            .then(function (value) { return _this.signatureDirectoryCreator.createDir(value); });
    };
    JobParserServiceImpl.prototype.handleSignatureRemoteWithTS = function (t) {
        var _this = this;
        return this.checkPermissions(t)
            .then(function (res) {
            var payload = {
                payload: lodash_1.cloneDeep(res),
                jobType: 'REMOTE_SIGNATURE_TS',
                priority: index_1.Priorities.HIGH,
                unique: false,
            };
            _this.jobsQueue.add(payload);
            return payload;
        })
            .then(function (value) { return _this.signatureDirectoryCreator.createDir(value); });
    };
    JobParserServiceImpl.prototype.handleMultipleVerification = function (t) {
        var _this = this;
        return this.checkPermissions(t).then(function (res) {
            var payload = {
                payload: {
                    type: res.type,
                    files: lodash_1.cloneDeep(res.files),
                },
                jobType: 'VERIFICATION',
                priority: index_1.Priorities.HIGH,
                unique: false,
            };
            _this.jobsQueue.add(payload);
            return payload;
        });
    };
    return JobParserServiceImpl;
}());
exports.JobParserServiceImpl = JobParserServiceImpl;
//# sourceMappingURL=jobParserService.js.map