"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var zlib = require('zlib');
var CryptoFlag;
(function (CryptoFlag) {
    CryptoFlag[CryptoFlag["CryptoFlagNone"] = 0] = "CryptoFlagNone";
    CryptoFlag[CryptoFlag["CryptoFlagCompression"] = 1] = "CryptoFlagCompression";
    CryptoFlag[CryptoFlag["CryptoFlagChecksum"] = 2] = "CryptoFlagChecksum";
    CryptoFlag[CryptoFlag["CryptoFlagHash"] = 4] = "CryptoFlagHash";
})(CryptoFlag || (CryptoFlag = {}));
;
var crc_tbl = [0x0000, 0x1081, 0x2102, 0x3183,
    0x4204, 0x5285, 0x6306, 0x7387,
    0x8408, 0x9489, 0xa50a, 0xb58b,
    0xc60c, 0xd68d, 0xe70e, 0xf78f];
var SimpleCrypt = /** @class */ (function () {
    function SimpleCrypt(logger, keyParts) {
        if (keyParts === void 0) { keyParts = [0x2f, 0xf9, 0xb9, 0x5c, 0xa4, 0xd4, 0x2d, 0x0c]; }
        this.logger = logger;
        this.keyParts = keyParts;
    }
    SimpleCrypt.prototype.decryptB64 = function (b64Data) {
        var data = Buffer.from(b64Data, 'base64');
        return this.decrypt(data);
    };
    SimpleCrypt.prototype.decrypt = function (cypher) {
        if (this.keyParts.length == 0) {
            this.logger.log("SimpleCrypt: ErrorNoKeySet");
            return Buffer.from('');
        }
        var ba = Buffer.from(cypher);
        if (cypher.length < 3) {
            return Buffer.from('');
        }
        var version = ba[0];
        if (version != 3) { //we only work with version
            this.logger.log("SimpleCrypt: ErrorUnknownVersion");
            return Buffer.from('');
        }
        var flags = ba[1];
        ba = ba.slice(2);
        var pos = 0;
        var cnt = ba.length;
        var lastChar = 0;
        while (pos < cnt) {
            var currentChar = ba[pos];
            ba[pos] = ba[pos] ^ lastChar ^ this.keyParts[pos % 8];
            lastChar = currentChar;
            ++pos;
        }
        //chop off the random number at the start
        ba = ba.slice(1);
        var integrityOk = true;
        if (flags & CryptoFlag.CryptoFlagChecksum) {
            if (ba.length < 2) {
                this.logger.log("SimpleCrypt: ErrorIntegrityFailed");
                return Buffer.from('');
            }
            var hi = ba[1] & 0x00FF;
            var lo = ba[0] << 8;
            var storedChecksum = hi | lo;
            ba = ba.slice(2);
            var qchecksum = this.qChecksum(Buffer.from(ba));
            integrityOk = (qchecksum == storedChecksum);
        }
        else if (flags & CryptoFlag.CryptoFlagHash) {
            // SKIP
            ba = ba.slice(20);
        }
        if (!integrityOk) {
            this.logger.log("SimpleCrypt: ErrorIntegrityFailed");
            return Buffer.from('');
        }
        if (flags & CryptoFlag.CryptoFlagCompression) {
            ba = this.qUncompress(ba);
        }
        this.logger.log("SimpleCrypt: AllOK");
        return ba;
    };
    SimpleCrypt.prototype.qChecksum = function (v) {
        var crc = 0xffff;
        var c;
        var len = v.length;
        var p = 0;
        while (p < len) {
            c = v[p];
            crc = ((crc >> 4) & 0x0fff) ^ crc_tbl[((crc ^ c) & 15)];
            c >>= 4;
            crc = ((crc >> 4) & 0x0fff) ^ crc_tbl[((crc ^ c) & 15)];
            p++;
        }
        return ~crc & 0xffff;
    };
    SimpleCrypt.prototype.qUncompress = function (data) {
        return zlib.inflateSync(data.slice(4));
    };
    return SimpleCrypt;
}());
exports.default = SimpleCrypt;
//# sourceMappingURL=SimpleCrypt.js.map